# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-07-10 03:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_dragonpay', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dragonpaytransaction',
            name='status',
            field=models.CharField(choices=[(b'S', b'Success'), (b'F', b'Failed'), (b'P', b'Pending'), (b'U', b'Unknown'), (b'R', b'Refund'), (b'K', b'Chargeback'), (b'V', b'Voided'), (b'A', b'Authorized'), (b'E', b'Expired')], default=b'P', max_length=1),
        ),
    ]
